;--------------------------------------------------------------------
;
;	rcen_prg.asm
;
;	Setzt auf dem 2-Prozessor-Board das RCEN-Fuse-Bit des
;	Slave Prozessors. Das geht nur im Parallel-Programmier-Modus.
;	Der Port B ist dazu 1 zu 1 mit dem Port B des Slaves verbunden.
;	(Master ist der AVR auf dem diese Software luft, Slave ist der
;	zweite AVR dessen RCEN Bit gesetzt wird). 
;
;	Insgesammt gilt folgendes Bit Mapping:
;
;	Master		Slave		Bemerkung
;	Port B 		Port B 		alle 8 Bit 1 zu 1 verbunden
;	PD 1		XTAL1		Quarzosz. Eingang des Slaves
;	PD 3		PD 3		WR Pin fr Parallelprog.
;	PD 5		PD 5		XA0 Pin fr Parallelprog.
;	PD 6		PD 6		XA1 Pin fr Parallelprog.
;
;	Original source writed by: Mr Herbert Dingfelder (Ver. 0.1)
;	Revision 1.1
;	Modified by: Serasidis Vasilis (Ver.0.2)
;	Home:  http://www.serasidis.gr
;	email:	info@serasidis.gr
;	Version: 0.3
;	
;	This revision can enable or disable the internal RC oscillator
;	of AT90S1200 and AT90S1200A types. Its dependent by the jumper JP.
;
;	modifications:
;	Added a jumper to PD4 and to the GND.
;
;	- If jumper its closed (0V DC) then the slave IC its programed with
;	internal RC oscillator ON.
;	- If jumper its open (5V DC) then the slave IC its programed with 
;	internal oscillator OFF
;		 
;	Added a resistor 220 ohm and a LED to PD2 to see the write progress time.
;	For the master IC  it can used the AT90S1200 or AT90S1200A with enabled RC oscillator
;	 or with any external crystal from 1MHz to 4 MHz. 
;
;+==================================================================================
;||
;||	Modifications made by Kyriakos Kontakos. (Ver.0.3)
;||	kkontak@hotmail.com
;||	22-10-2002
;||
;||	Modifications:
;||
;||	At the end of the process, it blinks the LED at two specific frequencies.
;||	- If the 1200 has been programmed for inernal oscillator (1MHz),
;||	 the LED blinks at 1Hz(slow) and 50% duty cycle.
;||	- If the 1200 has been programmed for external oscillator (usually at higher
;||	 frequencies than 1MHz), the LED blinks at 4Hz(fast) and 50% duty cycle.
;||
;||	Future plans:
;||
;||	Read back fuse to see if it has been programmed correctly.
;||	If error, leave LED constantly ON.
;||	For that a 4K7 resistor must be put in series with the jumper
;||	and the LED output must be moved to PD4. This way PD2 becomes free
;||	to use as OE signal for the slave MCU.
;||	Is it possible?????? 
;|| It seems that a more clever pin arrangment must be done to make this possible.
;||
;+==================================================================================
;
;
;	History:
;	
;	- V 0.3 By Kyriakos Kontakos. He change the LED indication. When you enable the RCEN bit
;		the LED its flashing slow and when you disable the RCEN bit the LED its
;		flashing fast. 
;
;	- V 0.2 By Serasidis Vasilis. Now can enable or disable the RCEN bit of AT90S1200(A)
;
;	- V 0.1 Start it from Herbert Dingfelder. This programmer can only enable the RCEN bit 
;		of AT90S1200(A)
;
;--------------------------------------------------------------------


.include "1200def.inc"	; Target ist ein 1200er

.def temp = r16		; Universalvariable fr dies und das
.def pa   = r17		; Einstellregister fr die Wartezeit bei Pause

; ------ Bitdefinitionen fr Port D ------
.equ XA0    = 5		;
.equ XA1    = 6
.equ JP     = 4		; For jumper if open (disable RCEN bit) 
				; or closed (enable RCEN bit)
.equ Busy   = 2		; For busy LED
.equ WR     = 3
.equ XTAL1  = 1
.equ prg12v = 0		; +12V fr Parallelprog wird von PD0 angest.


;---------------- Ports auf Ausgang schalten ------------------------

	ser temp
	out DDRB, temp
	ldi	temp,0b11101111 ;	;PD4 (SW) input, all others outputs	
	out DDRD, temp

;---------------- los gehts -----------------------------------------


	;Vcc einschwingen lassen

	;Startkonfiguration setzen
	
	sbi portD, Busy	; Power ON the busy led
	sbi portD, JP		; Enable PD4 pullup resistor
	cbi PORTD, XA0		; XA0, XA1 und XTAL auf 0 vorsetzen
	cbi PORTD, XA1 
	cbi PORTD, XTAL1
	sbi PORTD, WR		; WR ist aktiv low, deshalb mit 1 vorsetzen
	sbi PORTD, prg12v	; Reset des Slaves auf 0 setzen
	
	clr temp		; PORT B auf 0 vorsetzen
	out PORTB, temp

	;Der Reset der am Slave jetzt anliegt mu min. 100ns bestehen
	;bevor die 12V angelegt werden drfen

	ldi r18,15
InitDelay:
	ldi pa, 255
	rcall pause
	dec r18
	brne InitDelay

	;Parallelprog Modus aktivieren, dazu Reset auf 12V legen und 
	;min. 100ns warten (BS mu dabei auf 0 liegen aber das ist in
	;der benutzten Hardware eh auf GND hart verdrahtet
	
	cbi PORTD, prg12v	; 12V anlegen
	




;	sbic PIND,JP			;READ the jumper JP
;	rcall	EraseDevice		;If set to enable internal oscillator,
							; don't erase device.
	


	ldi pa, 255		; warten
	rcall pause
	
	;auf gehts mit den einzelnen Schritten

	sbi PORTD, XA1		; XA1 XA0 auf 10 setzen
	cbi PORTD, XA0		; (Enable Command loading)

	ldi pa, 255
	rcall pause
	
	ldi temp, 0b01000000	; 01000000 am Datenport anlegen
	out PORTB, temp

	ldi pa, 255
	rcall pause
	
	sbi PORTD, XTAL1	; positiven Puls auf XTAL1 ausgeben
	cbi PORTD, XTAL1

	ldi pa, 255
	rcall pause
	
	cbi PORTD, XA1		; XA1 XA0 auf 01 setzen
	sbi PORTD, XA0		; (enable data loading)
	
	ldi pa, 255
	rcall pause

;======	V--- These are modifications by K.Kontakos.	===========================

	ldi  temp,0b11011111	;if JP open, disable RCEN bit => ext oscillator

	sbis PIND,JP		;READ the jumper JP
	;rjmp	enable
	;ldi  temp,0b11011111	;if JP open, disable RCEN bit
	;rjmp	cont		;if JP closed, enable RCEN bit => int oscillator
	
enable:	ldi temp, 0b11011110	; 1111 1110 als Daten anlegen 
				; RCEN Fuse ist Bit0 (0=RC Osziallator aktiviert)
				; Achtung: Bit5 ist Serial-programing-enabled SPIEN
				;	und mu programmiert bleiben, d.h auf 0 !!!
cont:	mov r19,temp

;======	End of modifications.	==============================================

	out PORTB, temp		; Daten rausschreiben
	
	ldi pa, 255
	rcall pause

	sbi PORTD, XTAL1	; positiven Puls auf XTAL1 ausgeben
	cbi PORTD, XTAL1

	ldi pa, 255
	rcall pause
	
	cbi PORTD, WR		; Schreibvorgang starten durch WR auf 0
	ldi pa, 26
	rcall pause		; fr 1.5 ms warten
	sbi PORTD, WR
		
	ldi pa, 255
	rcall pause
	
	sbi PORTD, prg12v	; 12V Programmierspannung vom Reset wieder weg	

	cbi portD, Busy	; Power OFF the busy led

;======	V--- These are modifications by K.Kontakos.	===========================

	ldi pa, 255
	rcall pause
fertig:	
	ldi r18,250		;If set 1200 for int=1MHz, blink LED @ 1Hz
	sbrc r19,0
	ldi r18,60		;If set 1200 for ext=?, blink LED @ 3Hz
WaitON:
	sbi portD, Busy	; Power ON the busy led

	ldi pa,4
	rcall pause

	ldi pa,4
	rcall pause

	;rcall Delay1ms
	;rcall Delay1ms

	dec r18
	brne WaitON
	
	ldi r18,250
	sbrc r19,0
	ldi r18,60
WaitOFF:
	cbi portD, Busy	; Power OFF the busy led

	ldi pa,4
	rcall pause

	ldi pa,4
	rcall pause

	;rcall Delay1ms
	;rcall Delay1ms

	dec r18
	brne WaitOFF

	rjmp fertig

;-------------------------- Erase device ----------------------------------------

;This subroutin is currently not used but its free to use it if necessary.

EraseDevice:

	ldi pa, 255		; warten
	rcall pause
	
	;auf gehts mit den einzelnen Schritten

	sbi PORTD, XA1		; XA1 XA0 auf 10 setzen
	cbi PORTD, XA0		; (Enable Command loading)

	ldi pa, 255
	rcall pause
	
	ldi temp, 0b10000000	; 10000000 chip erase
	out PORTB, temp

	ldi pa, 255
	rcall pause
	
	sbi PORTD, XTAL1	; positiven Puls auf XTAL1 ausgeben
	cbi PORTD, XTAL1

	ldi pa, 255
	rcall pause

	cbi PORTD, WR		; Schreibvorgang starten durch WR auf 0
	ldi pa, 26
	rcall pause		; fr 1.5 ms warten
	sbi PORTD, WR

	ret

;======	End of modifications.	==============================================
	
;-------------------------- pause --------------------------------------------
; abhngig vom Inhalt von reg. pa wird entspr. lange gewartet
; einmal pa entspricht einmal berlaufen des 8-Bit Timers mit
; einem Vorteiler von 1, d.h. t= (1/fq) * 1 * 256
; (der Master luft auf einen 4.433 MHz Quarz)

pause:
		
pa_loop:
	; Vorteiler fr den Timer auf 1 setzen
	ldi temp, 0b00000001
	out tccr0, temp

	; das berlaufbit des Counters zurcksetzen,
	in temp, tifr
	sbr temp, exp2(tov0)
	out tifr, temp

	; Den Zhler mit Null vorsetzen
	clr temp
	out tcnt0, temp

	; wenn tcnt0 berluft wird tov0 gesetzt
cwait:	in temp, tifr
	sbrs temp, tov0
	rjmp cwait

	dec pa			; jetzt ist der Timer einmal bergelaufen, 
	brne pa_loop		; pa runterzhlen, wenn noch nicht auf 0
				; dann nochmal hoch und den Timer neu aufziehen
	ret
	